package ehu.student.collections;

import java.util.ArrayList;

/**
 * Una clase de objetos capaces de crear
 * listas con numeros primos.
 */
public class CalculadorNumerosPrimos {

    /**
     * Devuelve una nueva lista con los "n" primeros numeros primos.
     */
    public ArrayList<Integer> crearListaDePrimos(int n){
        final ArrayList<Integer> lista = new ArrayList<Integer>();
                
        /*
         * Anyadir a la lista "n" numeros primos.
         * 
         * Se examinan valores enteros consecutivos 2, 3, 4, 5, 6...
         * El valor examinado se anyade a la lista si y solo si es primo.
         * Este proceso se termina cuando la lista tiene "n" elementos. 
         */
        int valorAExaminar = 2;
        while (lista.size() < n){ // mientras el numero de elementos sea < n
            /*
             * Si el valor a examinar es primo se anyade a la lista.
             * 
             * La lista contiene todos los numeros primos que son
             * menores que el valor a examinar.
             * POR LO TANTO: el valor a examinar es primo si no es
             * divisible por ninguno de los elementos de la lista.
             */
            if (!esDivisiblePorAlgunElementoDeLaLista(valorAExaminar, lista)){
                lista.add(valorAExaminar);
            }                   
            valorAExaminar++;
        }
        return lista;
    }

    /**
     * Dada una lista "lst" de numeros enteros y un entero "x",
     * devuelve true si "x" es divisible por alguno de los elementos de 
     * la lista; false, en caso contrario.
     */
    public boolean esDivisiblePorAlgunElementoDeLaLista(int x, 
                                                        ArrayList<Integer> lst) 
    {
        for (Integer v: lst){ //para cada elemento "v" de "lst"
            if (x % v == 0) return true; //si v es es divisible, devuelve true!
        }
        return false; //NO es divisible: devuelve false
    }
        
    /**
     * Devuelve true si "n" es primo.
     */
    public boolean esPrimo(int n){
        ArrayList<Integer> listaDePrimos = crearListaDePrimos(n);
        boolean esPrimo = listaDePrimos.contains(n);
        return esPrimo;
    }
}
